//
// System.Drawing.Pen.cs
//
// Author:
//   Miguel de Icaza (miguel@ximian.com)
//   Alexandre Pigolkine (pigolkine@gmx.de)
//   Duncan Mak (duncan@ximian.com)
//
// (C) Ximian, Inc.  http://www.ximian.com
//

using System;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;

namespace System.Drawing {

	public sealed class Pen : MarshalByRefObject, ICloneable, IDisposable {
		internal IntPtr nativeObject;

                internal Pen (IntPtr p)
                {
                        nativeObject = p;
                }

		public Pen (Brush brush) : this (brush, 1.0F)
		{
		}

		public Pen (Color color) : this (color, 1.0F)
		{
		}

		public Pen (Brush brush, float width)
		{
			int pen;
			GDIPlus.GdipCreatePen2 (brush.nativeObject, width, Unit.UnitWorld, out pen);
			nativeObject = (IntPtr) pen;
		}

		public Pen (Color color, float width)
		{
			int pen;
			GDIPlus.GdipCreatePen1 (color.ToArgb (), width, Unit.UnitWorld, out pen);
			nativeObject = (IntPtr)pen;
		}

		//
		// Properties
		//
		public PenAlignment Alignment {
			get {
				PenAlignment retval;
                                GDIPlus.GdipGetPenMode (nativeObject, out retval);

                                return retval;
                        }

			set {
				GDIPlus.GdipSetPenMode (nativeObject, value);
			}
		}

		public Brush Brush {
			get {
                                IntPtr retval;
                                GDIPlus.GdipGetPenBrushFill (nativeObject, out retval);
                                BrushType type;
                                GDIPlus.GdipGetBrushType (retval, out type);
                                
                                return Brush.CreateBrush (retval, type);
                        }

			set {
                                GDIPlus.GdipSetPenBrushFill (nativeObject, value.nativeObject);
			}
		}

		public Color Color {
			get {
				int argb;
                                GDIPlus.GdipGetPenColor (nativeObject, out argb);

                                return Color.FromArgb (argb);
			}

			set {
				GDIPlus.GdipSetPenColor (nativeObject, value.ToArgb ());
			}
		}

                public float [] CompoundArray {
                        get {
                                throw new NotImplementedException ();
//                                 int count;
//                                 GDIPlus.GdipGetPenCompoundArrayCount (nativeObject, out count);

//                                 IntPtr tmp = Marshal.AllocHGlobal (8 * count);
//                                 GDIPlus.GdipGetPenCompoundArray (nativeObject, out tmp, out count);

//                                 float [] retval = new float [count];
//                                 Marshal.Copy (tmp, retval, 0, count);

//                                 Marshal.FreeHGlobal (tmp);

//                                 return retval;
                        }

                        set {
                                throw new NotImplementedException ();                                
//                                 int length = value.Length;
//                                 IntPtr tmp = Marshal.AllocHGlobal (8 * length);
//                                 Marshal.Copy (value, 0, tmp, length);
//                                 GDIPlus.GdipSetPenCompoundArray (nativeObject, tmp, length);

//                                 Marshal.FreeHGlobal (tmp);
                        }
                }

                [MonoTODO]
                public CustomLineCap CustomEndCap {
                        get {
                                throw new NotImplementedException ();
                        }

                        set {
                                throw new NotImplementedException ();                                
                        }
                }

                [MonoTODO]
                public CustomLineCap CustomStartCap {

                        get {
                                throw new NotImplementedException ();                                
                        }

                        set {
                                throw new NotImplementedException ();                                
                        }
                }

                public DashCap DashCap {

                        get {
                                DashCap retval;
                                GDIPlus.GdipGetPenDashCap (nativeObject, out retval);

                                return retval;
                        }

                        set {
                                GDIPlus.GdipSetPenDashCap (nativeObject, value);
                        }
                }

                public float DashOffset {

                        get {
                                float retval;
                                GDIPlus.GdipGetPenDashOffset (nativeObject, out retval);

                                return retval;
                        }

                        set {
                                GDIPlus.GdipSetPenDashOffset (nativeObject, value);
                        }
                }

                public float [] DashPattern {
                        get {
                                int count;
                                GDIPlus.GdipGetPenDashCount (nativeObject, out count);

                                IntPtr tmp = Marshal.AllocHGlobal (8 * count);
                                GDIPlus.GdipGetPenDashArray (nativeObject, out tmp, out count);

                                float [] retval = new float [count];
                                Marshal.Copy (tmp, retval, 0, count);

                                Marshal.FreeHGlobal (tmp);

                                return retval;
                        }

                        set {
                                int length = value.Length;
                                IntPtr tmp = Marshal.AllocHGlobal (8 * length);
                                Marshal.Copy (value, 0, tmp, length);
                                GDIPlus.GdipSetPenDashArray (nativeObject, tmp, length);

                                Marshal.FreeHGlobal (tmp);
                        }
                }

		public DashStyle DashStyle {
			get {
				DashStyle retval;
                                GDIPlus.GdipGetPenDashStyle (nativeObject, out retval);

                                return retval;
			}

			set {
				GDIPlus.GdipSetPenDashStyle (nativeObject, value);
			}
		}

		public LineCap StartCap {
			get {
                                throw new NotImplementedException ();
// 				LineCap retval;
//                                 GDIPlus.GdipGetPenStartCap (nativeObject, out retval);

//                                 return retval;
			}

			set {
                                throw new NotImplementedException ();                                
// 				GDIPlus.GdipSetPenStartCap (nativeObject, value);
			}
		}
 
		public LineCap EndCap {
			get {
                                throw new NotImplementedException ();                                
// 				LineCap retval;
//                                 GDIPlus.GdipGetPenEndCap (nativeObject, out retval);

//                                 return retval;
			}

			set {
                                throw new NotImplementedException ();                                
// 				GDIPlus.GdipSetPenEndCap (nativeObject, value);
			}
		}
 
                public LineJoin LineJoin {

                        get {
                                LineJoin result;
                                GDIPlus.GdipGetPenLineJoin (nativeObject, out result);
                                return result;
                        }

                        set {
                                GDIPlus.GdipSetPenLineJoin (nativeObject, value);
                        }
                                
                }

                public float MiterLimit {

                        get {
                                float result;
                                GDIPlus.GdipGetPenMiterLimit (nativeObject, out result);
                                return result;
                        }

                        set {
                                GDIPlus.GdipSetPenMiterLimit (nativeObject, value);
                        }
                                
                }

                public Matrix Transform {

                        get {
                                Matrix result;
                                GDIPlus.GdipGetPenTransform (nativeObject, out result);
                                return result;
                        }

                        set {
                                GDIPlus.GdipSetPenTransform (nativeObject, value);
                        }
                }

		public float Width {
			get {
				float f;
                                GDIPlus.GdipGetPenWidth (nativeObject, out f);
                                return f;
			}
			set {
				GDIPlus.GdipSetPenWidth (nativeObject, value);
			}
		}

		public object Clone ()
		{
                        IntPtr ptr;
                        GDIPlus.GdipClonePen (nativeObject, out ptr);

                        return new Pen (ptr);
		}

		public void Dispose ()
		{
			Dispose (true);
			System.GC.SuppressFinalize (this);
		}

		void Dispose (bool disposing)
		{
                        GDIPlus.GdipDeletePen (nativeObject);
		}

		~Pen ()
		{
			Dispose (false);
		}

                public void MultiplyTransform (Matrix matrix)
                {
                        MultiplyTransform (matrix, MatrixOrder.Prepend);
                }

                public void MultiplyTransform (Matrix matrix, MatrixOrder order)
                {
                        GDIPlus.GdipMultiplyPenTransform (nativeObject, matrix.nativeMatrix, order);
                }

                public void ResetTransform ()
                {
                        GDIPlus.GdipResetPenTransform (nativeObject);
                }

                public void RotateTransform (float angle)
                {
                        RotateTransform (angle, MatrixOrder.Prepend);
                }

                public void RotateTransform (float angle, MatrixOrder order)
                {
                        GDIPlus.GdipRotatePenTransform (nativeObject, angle, order);
                }

                public void ScaleTransform (float sx, float sy)
                {
                        ScaleTransform (sx, sy, MatrixOrder.Prepend);
                }

                public void ScaleTransform (float sx, float sy, MatrixOrder order)
                {
                        GDIPlus.GdipScalePenTransform (nativeObject, sx, sy, order);
                }

                public void SetLineCap (LineCap startCap, LineCap endCap, DashCap dashCap)
                {
                        // GDIPlus.GdipSetLineCap197819 (nativeObject, startCap, endCap, dashCap);
                }

                public void TranslateTransform (float dx, float dy)
                {
                        TranslateTransform (dx, dy, MatrixOrder.Prepend);
                }

                public void TranslateTransform (float dx, float dy, MatrixOrder order)
                {
                        GDIPlus.GdipTranslatePenTransform (nativeObject, dx, dy, order);
                }
	}
}
